@@ -166,8 +166,6 @@ xmlCharEncoding
 	xmlDetectCharEncoding		(const unsigned char* in,
 					 int len);
 
-int	xmlCheckUTF8			(const unsigned char *utf);
-
 int	xmlCharEncOutFunc		(xmlCharEncodingHandler *handler,
 					 xmlBufferPtr out,
 					 xmlBufferPtr in);
@@ -180,6 +178,20 @@ int	xmlCharEncFirstLine		(xmlCharEncodingHandler *handler,
 					 xmlBufferPtr in);
 int	xmlCharEncCloseFunc		(xmlCharEncodingHandler *handler);
 
+/*
+ * Export a few useful functions
+ */
+int	UTF8Toisolat1			(unsigned char* out,
+					 int *outlen,
+					 const unsigned char* in,
+					 int *inlen);
+int	isolat1ToUTF8			(unsigned char* out,
+					 int *outlen,
+					 const unsigned char* in,
+					 int *inlen);
+int	xmlCheckUTF8			(const unsigned char *utf);
+int	xmlUTF8Strlen			(const unsigned char *utf);
+
 #ifdef __cplusplus
 }
 #endif
