@@ -54,8 +54,8 @@ typedef enum {
     XML_NAMESPACE_DECL=		18,
     XML_XINCLUDE_START=		19,
     XML_XINCLUDE_END=		20
-#ifdef LIBXML_SGML_ENABLED
-   ,XML_SGML_DOCUMENT_NODE=	21
+#ifdef LIBXML_DOCB_ENABLED
+   ,XML_DOCB_DOCUMENT_NODE=	21
 #endif
 } xmlElementType;
 
@@ -680,17 +680,24 @@ void		xmlElemDump		(FILE *f,
 					 xmlNodePtr cur);
 int		xmlSaveFile		(const char *filename,
 					 xmlDocPtr cur);
+int		xmlSaveFormatFile	(const char *filename,
+					 xmlDocPtr cur,
+					 int format);
 void		xmlNodeDump		(xmlBufferPtr buf,
 					 xmlDocPtr doc,
 					 xmlNodePtr cur,
 					 int level,
 					 int format);
 
-/* This one is exported from xmlIO.h
+/* These are exported from xmlIO.h
  
 int		xmlSaveFileTo		(xmlOutputBuffer *buf,
 					 xmlDocPtr cur,
 					 const char *encoding);
+int             xmlSaveFormatFileTo     (xmlOutputBuffer *buf,
+                                         xmlDocPtr cur,
+                                         const char *encoding,
+                                         int format);
  */ 					 
 
 int		xmlSaveFileEnc		(const char *filename,
