@@ -20,10 +20,10 @@ extern "C" {
 #ifndef LIBXML2_COMPILING_MSCCDEF
 extern void xmlCheckVersion(int version);
 #endif /* LIBXML2_COMPILING_MSCCDEF */
-#define LIBXML_DOTTED_VERSION "2.3.7"
-#define LIBXML_VERSION 20307
-#define LIBXML_VERSION_STRING "20307"
-#define LIBXML_TEST_VERSION xmlCheckVersion(20307);
+#define LIBXML_DOTTED_VERSION "2.3.8"
+#define LIBXML_VERSION 20308
+#define LIBXML_VERSION_STRING "20308"
+#define LIBXML_TEST_VERSION xmlCheckVersion(20308);
 
 /*
  * Whether the trio support need to be configured in
@@ -62,13 +62,13 @@ extern void xmlCheckVersion(int version);
 #endif
 
 /*
- * Whether the Docbook support is configured in
-#if @WITH_SGML@
-#define LIBXML_SGML_ENABLED
-#else
-#define LIBXML_SGML_DISABLED
-#endif
+ * Whether the SGML Docbook support is configured in
  */
+#if 1
+#define LIBXML_DOCB_ENABLED
+#else
+#define LIBXML_DOCB_DISABLED
+#endif
 
 /*
  * Whether XPath is configured in
