@@ -48,7 +48,9 @@ typedef enum {
     XPTR_SYNTAX_ERROR,
     XPTR_RESOURCE_ERROR,
     XPTR_SUB_RESOURCE_ERROR,
-    XPATH_UNDEF_PREFIX_ERROR
+    XPATH_UNDEF_PREFIX_ERROR,
+    XPATH_ENCODING_ERROR,
+    XPATH_INVALID_CHAR_ERROR
 } xmlXPathError;
 
 /*
